@@ -1,10 +1,11 @@
+import json
 import click
 import pprint
 import tarfile
+import warnings
+import traceback
 from pathlib import Path
 from collections.abc import Iterable
-import warnings
-import json
 
 from benchopt.config import set_setting
 from benchopt.config import get_setting
@@ -21,7 +22,7 @@ from benchopt.config import GLOBAL_CONFIG_FILE_MODE
 from benchopt.utils.dynamic_modules import _load_class_from_module
 from benchopt.utils.shell_cmd import _run_shell_in_conda_env
 from benchopt.utils.terminal_output import colorify
-from benchopt.utils.terminal_output import RED, GREEN, TICK, CROSS
+from benchopt.utils.terminal_output import BLUE, RED, GREEN, TICK, CROSS
 
 
 ARCHIVE_ELEMENTS = [
@@ -72,6 +73,11 @@ def clean(benchmark, token=None, filename='all'):
                 json_file.pop(f"{filename}.{ext}", None)
                 with open(json_path, "w") as cache_run:
                     json.dump(json_file, cache_run)
+    # Delete slurm files
+    slurm_folder = benchmark.get_slurm_folder()
+    if slurm_folder.exists():
+        print(f"rm -rf {slurm_folder}")
+        rm_folder(slurm_folder)
     # Delete cache files
     print("Clear joblib cache")
     benchmark.mem.clear(warn=False)
@@ -225,10 +231,10 @@ def print_info(cls_name_list, cls_list, env_name=None, verbose=False):
             if hasattr(cls, 'requirements') and cls.requirements:
                 print("> requirements:")
                 packages = cls.requirements
-                pip_packages = [pkg[4:] for pkg in packages
-                                if pkg.startswith('pip:')]
+                pip_packages = [pkg[5:] for pkg in packages
+                                if pkg.startswith('pip::')]
                 conda_packages = [pkg for pkg in packages
-                                  if not pkg.startswith('pip:')]
+                                  if not pkg.startswith('pip::')]
                 if len(conda_packages) > 0:
                     print("    conda install -c conda-forge "
                           f"{' '.join(conda_packages)}")
@@ -299,8 +305,8 @@ def info(benchmark, solver_names, dataset_names, env_name='False',
     print(f"Info regarding the benchmark '{benchmark.name}'")
 
     # validate solvers and datasets
-    benchmark.validate_dataset_patterns(dataset_names)
-    benchmark.validate_solver_patterns(solver_names)
+    benchmark.check_dataset_patterns(dataset_names)
+    benchmark.check_solver_patterns(solver_names)
 
     # get solvers and datasets in the benchmark
     all_solvers = benchmark.get_solvers()
@@ -457,6 +463,36 @@ def check_install(benchmark, module_filename, base_class_name):
 
     # Get class to check
     klass = _load_class_from_module(
-        module_filename, base_class_name, benchmark.benchmark_dir
+         benchmark.benchmark_dir, module_filename, base_class_name
     )
     klass.is_installed(raise_on_not_installed=True)
+
+
+@helpers.command(
+    help="Ensure that the data of a list of Datasets are available\n\n"
+    "The names of the datasets to check/download are specified as a list.",
+    hidden=True
+)
+@click.argument('benchmark', default=Path.cwd(), type=click.Path(exists=True),
+                shell_complete=complete_benchmarks)
+@click.option('--dataset', '-d', 'dataset_names',
+              metavar="<dataset_name>", multiple=True, type=str,
+              help="Check data from the dataset <dataset_name>.",
+              shell_complete=complete_datasets)
+def check_data(benchmark, dataset_names):
+    warnings.simplefilter("ignore")
+
+    # benchmark
+    benchmark = Benchmark(benchmark)
+    datasets = benchmark.check_dataset_patterns(dataset_names, class_only=True)
+
+    # Get class to check
+    print("Loading data:")
+    for cls in datasets:
+        print(f"- {colorify(cls.name, BLUE)} ...", end='', flush=True)
+        try:
+            cls.get_instance().get_data()
+            print(colorify(f" {TICK}", GREEN))
+        except Exception:
+            print(colorify(f" {CROSS}", RED))
+            traceback.print_exc()
