@@ -4,6 +4,7 @@ import warnings
 from pathlib import Path
 
 from benchopt.benchmark import Benchmark
+from benchopt.config import get_setting
 from benchopt.cli.completion import complete_solvers
 from benchopt.cli.completion import complete_datasets
 from benchopt.cli.completion import complete_benchmarks
@@ -51,16 +52,18 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
         "max_runs",
         "n_repetitions",
         "timeout",
+        "no_timeout",
         "n_jobs",
         "slurm",
+        "collect",
         "plot",
+        "display",
         "html",
         "pdb",
         "profile",
         "env_name",
+        "no_cache",
         "output",
-        "objective_filter",
-        "old_objective_filter",
     ]
     return [cli_kwargs[name] for name in return_names]
 
@@ -73,17 +76,11 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
 )
 @click.argument('benchmark', default=Path.cwd(), type=click.Path(exists=True),
                 shell_complete=complete_benchmarks)
-@click.option('--objective-filter',
-              metavar='<objective_filter>', multiple=True, type=str,
-              help="Deprecated alias for `--objective`.")
 @click.option('--objective', '-o',
               metavar='<objective_filter>', multiple=True, type=str,
               help="Select the objective based on its parameters, with the "
               "syntax `objective[parameter=value]`. This can be used to only "
               "include one set of parameters. ")
-@click.option('--old-objective-filter', '-p',
-              multiple=True, type=str,
-              help="Deprecated alias for --objective_filter/-o.")
 @click.option('--solver', '-s',
               metavar="<solver_name>", multiple=True, type=str,
               help="Include <solver_name> in the benchmark. By default, all "
@@ -120,20 +117,36 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
               help='Maximal number of runs for each solver. This corresponds '
               'to the number of points in the time/accuracy curve.')
 @click.option('--n-repetitions', '-r',
-              metavar='<int>', default=1, show_default=True, type=int,
+              metavar='<int>', default=None, show_default=True, type=int,
               help='Number of repetitions that are averaged to estimate the '
               'runtime.')
 @click.option('--timeout',
-              metavar="<int>", default=100, show_default=True, type=int,
-              help='Timeout a solver when run for more than <timeout> seconds')
+              default=None, show_default=True, type=str,
+              help='Stop a solver when run for more than <timeout> seconds. '
+              'The syntax 10h or 10m can be used to denote 10 hours or '
+              'minutes respectively. Not compatible with the --no-timeout '
+              'option.')
+@click.option('--no-timeout',
+              is_flag=True,
+              help='If set, prevent solvers from stopping after running for '
+              'a long time. Not compatible with the --timeout option.')
+@click.option('--collect',
+              is_flag=True,
+              help='If set, this run will only collect results which are '
+              'already available in the cache. This flag allows to collect '
+              'results while all the solvers are not finished yet.')
 @click.option('--config', 'config_file', default=None,
               shell_complete=complete_config_files,
               help="YAML configuration file containing benchmark options.")
 @click.option('--plot/--no-plot', default=True,
-              help="Whether or not to plot the results. Default is True.")
+              help="Whether or not to create plots from the results. "
+              "Default is True.")
+@click.option('--display/--no-display', default=True,
+              help="Whether or not to display the plot on the screen. "
+              "Default is True.")
 @click.option('--html/--no-html', default=True,
-              help="Whether to display the plot as HTML report or matplotlib"
-              "figures, default is True.")
+              help="If set to True (default), render the results as an HTML "
+              "page, otherwise create matplotlib figures, saved as PNG.")
 @click.option('--pdb',
               is_flag=True,
               help="Launch a debugger if there is an error. This will launch "
@@ -158,6 +171,10 @@ def _get_run_args(cli_kwargs, config_file_kwargs):
               help="Run the benchmark in the conda environment "
               "named <env_name>. To install the required solvers and "
               "datasets, see the command `benchopt install`.")
+@click.option('--no-cache',
+              is_flag=True,
+              help='If set, disable the cache on disk for the run. Note that '
+              'this makes the run less tolerant to errors, use with caution.')
 @click.option("--output", default="None", type=str,
               help="Filename for the result output. "
               "If given, the results will "
@@ -175,30 +192,32 @@ def run(config_file=None, **kwargs):
     else:
         config = {}
 
-    # XXX - Remove old and deprecated objective filters in version 1.3
     (
         benchmark, solver_names, forced_solvers, dataset_names,
-        objective_filters, max_runs, n_repetitions, timeout, n_jobs, slurm,
-        plot, html, pdb, do_profile, env_name, output,
-        deprecated_objective_filters, old_objective_filters
+        objective_filters, max_runs, n_repetitions, timeout, no_timeout,
+        n_jobs, slurm, collect, plot, display, html, pdb, do_profile,
+        env_name, no_cache, output_name
     ) = _get_run_args(kwargs, config)
 
-    if len(old_objective_filters):
-        warnings.warn(
-            'Using the -p option is deprecated, use -o instead',
-            FutureWarning,
+    # If --no-timeout is set and --timeout is not, skip these blocks
+    # and keep timeout = None
+    if no_timeout and timeout is not None:
+        raise click.BadParameter(
+            'You cannot specify both --timeout and --no-timeout options.'
         )
-        objective_filters = old_objective_filters
 
-    if len(deprecated_objective_filters):
-        warnings.warn(
-            'Using the --objective-filters option is deprecated, '
-            'use --objective instead', FutureWarning
-        )
-        objective_filters = deprecated_objective_filters
+    if not no_timeout:
+        if timeout is None:
+            timeout = get_setting('default_timeout')
+        else:
+            try:
+                timeout = int(float(timeout))
+            except ValueError:  # already under string format
+                import pandas as pd
+                timeout = pd.to_timedelta(timeout).total_seconds()
 
     # Create the Benchmark object
-    benchmark = Benchmark(benchmark)
+    benchmark = Benchmark(benchmark, no_cache=no_cache)
 
     if benchmark.min_version is not None:
         from packaging.version import parse
@@ -223,7 +242,7 @@ def run(config_file=None, **kwargs):
             print("Running on SLURM")
             set_slurm_launch()
 
-        from benchopt.runner import run_benchmark
+        from benchopt.runner import _run_benchmark
 
         if do_profile:
             from benchopt.utils.profiling import use_profile
@@ -234,21 +253,25 @@ def run(config_file=None, **kwargs):
         objective.is_installed(raise_on_not_installed=True)
 
         # Check that the dataset/solver patterns match actual dataset
-        benchmark.validate_dataset_patterns(dataset_names)
-        benchmark.validate_objective_filters(objective_filters)
+        datasets = benchmark.check_dataset_patterns(dataset_names)
+        objectives = benchmark.check_objective_filters(objective_filters)
+
         # pyyaml returns tuples: make sure everything is a list
-        benchmark.validate_solver_patterns(
-            list(solver_names) + list(forced_solvers)
+        if isinstance(solver_names, dict):
+            solver_names = [solver_names]
+        elif isinstance(solver_names, tuple):
+            solver_names = list(solver_names)
+        solvers = benchmark.check_solver_patterns(
+            solver_names + list(forced_solvers)
         )
 
-        run_benchmark(
-            benchmark, solver_names, forced_solvers,
-            dataset_names=dataset_names,
-            objective_filters=objective_filters,
+        _run_benchmark(
+            benchmark, solvers, forced_solvers,
+            datasets=datasets, objectives=objectives,
             max_runs=max_runs, n_repetitions=n_repetitions,
             timeout=timeout, n_jobs=n_jobs, slurm=slurm,
-            plot_result=plot, html=html, pdb=pdb,
-            output=output
+            plot_result=plot, display=display, html=html,
+            collect=collect, pdb=pdb, output_name=output_name
         )
 
         print_stats()  # print profiling stats (does nothing if not profiling)
@@ -310,21 +333,25 @@ def run(config_file=None, **kwargs):
             f"and version in env {env_name} ({benchopt_version}) differ")
 
     # run the command in the conda env
-    solvers_option = ' '.join([f"-s '{s}'" for s in solver_names])
-    forced_solvers_option = ' '.join([f"-f '{s}'" for s in forced_solvers])
-    datasets_option = ' '.join([f"-d '{d}'" for d in dataset_names])
-    objective_option = ' '.join([f"-o '{o}'" for o in objective_filters])
+    solvers_option = ' '.join([f'-s "{s}"' for s in solver_names])
+    forced_solvers_option = ' '.join([f'-f "{s}"' for s in forced_solvers])
+    datasets_option = ' '.join([f'-d "{d}"' for d in dataset_names])
+    objective_option = ' '.join([f'-o "{o}"' for o in objective_filters])
     cmd = (
         rf"benchopt run --local {benchmark.benchmark_dir} "
         rf"--n-repetitions {n_repetitions} "
-        rf"--max-runs {max_runs} --timeout {timeout} "
+        rf"--max-runs {max_runs} "
+        rf"{f'--timeout {timeout} ' if timeout is not None else ''}"
+        rf"{'--no-timeout ' if no_timeout else ''} "
         rf"--n-jobs {n_jobs} {'--slurm' if slurm else ''} "
         rf"{solvers_option} {forced_solvers_option} "
         rf"{datasets_option} {objective_option} "
         rf"{'--plot' if plot else '--no-plot'} "
+        rf"{'--display' if display else '--no-display'} "
         rf"{'--html' if html else '--no-html'} "
-        rf"{'--pdb' if pdb else ''} "
-        rf"--output {output}"
+        rf"{'--pdb ' if pdb else ''}"
+        rf"{'--profile ' if do_profile else ''}"
+        rf"--output {output_name}"
         .replace('\\', '\\\\')
     )
     raise SystemExit(_run_shell_in_conda_env(
@@ -369,6 +396,9 @@ def run(config_file=None, **kwargs):
               shell_complete=complete_config_files,
               help="YAML configuration file containing benchmark options, "
               "whose solvers and datasets will be installed.")
+@click.option('--download', is_flag=True,
+              help="If this flag is set, call `Dataset.get_data` for all "
+              "datasets, to make sure the data are present on the system.")
 @click.option('--env', '-e', 'env_name',
               flag_value='True', type=str, default='False',
               help="Install all requirements in a dedicated "
@@ -392,10 +422,14 @@ def run(config_file=None, **kwargs):
               help="If this flag is set, no confirmation will be asked "
               "to the user to install requirements in the current environment."
               " Useless with options `-e/--env` or `--env-name`.")
+@click.option('--gpu', is_flag=True, default=False,
+              help="Use this flag to install requirements['gpu'] for solvers "
+                   "and datasets that have different requirements for GPU and "
+                   "CPU.")
 def install(
         benchmark, minimal, solver_names, dataset_names, config_file=None,
         force=False, recreate=False, env_name='False', confirm=False,
-        quiet=False):
+        quiet=False, download=False, gpu=False):
 
     if config_file is not None:
         with open(config_file, "r") as f:
@@ -407,10 +441,8 @@ def install(
             forced_solvers = config.get("force-solver", tuple())
             solver_names = list(set(solver_names).union(set(forced_solvers)))
 
-    # Check that the dataset/solver patterns match actual dataset
+    # Instantiate the benchmark
     benchmark = Benchmark(benchmark)
-    benchmark.validate_dataset_patterns(dataset_names)
-    benchmark.validate_solver_patterns(solver_names)
 
     # Get a list of all conda envs
     default_conda_env, conda_envs = list_conda_envs()
@@ -460,19 +492,33 @@ def install(
         # create environment if necessary
         create_conda_env(env_name, recreate=recreate, quiet=quiet)
 
+    # List solver and datasets classes to install
+    if len(dataset_names) == 0 and len(solver_names) > 0:
+        datasets = []
+    else:
+        datasets = benchmark.check_dataset_patterns(
+            dataset_names, class_only=True
+        )
+    if len(solver_names) == 0 and len(dataset_names) > 0:
+        solvers = []
+    else:
+        solvers = benchmark.check_solver_patterns(
+            solver_names, class_only=True
+        )
+
     # install requirements
     print("# Install", flush=True)
     benchmark.install_all_requirements(
-        include_solvers=solver_names, include_datasets=dataset_names,
+        include_solvers=solvers, include_datasets=datasets,
         minimal=minimal, env_name=env_name, force=force, quiet=quiet,
+        download=download, gpu=gpu,
     )
 
 
 @main.command(
     help="Test a benchmark for benchopt. The benchmark must feature a "
     "simulated dataset to test for all solvers. For more info about the "
-    "simulated dataset configurations, see"
-    "benchopt.github.io/how.html#example-of-parametrized-simulated-dataset",
+    "benchmark tests configuration and requirements, see :ref:`test_config`.",
     context_settings=dict(ignore_unknown_options=True)
 )
 @click.argument('benchmark', default=Path.cwd(), type=click.Path(exists=True),
@@ -498,7 +544,7 @@ def test(benchmark, env_name, pytest_args):
 
     env_option = ''
     if env_name is not None:
-        create_conda_env(env_name, with_pytest=True)
+        create_conda_env(env_name, pytest=True)
         if _run_shell_in_conda_env("pytest --version", env_name=env_name) != 0:
             raise ModuleNotFoundError(
                 f"pytest is not installed in conda env {env_name}.\n"
