@@ -1,8 +1,8 @@
 import time
 import math
 
-# Possible stop strategies
-STOPPING_STRATEGIES = ['iteration', 'tolerance', 'callback']
+# Possible curve sampling strategies
+SAMPLING_STRATEGIES = ['iteration', 'tolerance', 'callback', 'run_once']
 
 EPS = 1e-10
 PATIENCE = 3
@@ -18,8 +18,23 @@ RHO = 1.5
 RHO_INC = 1.2  # multiplicative update if rho is too small
 
 
+COMMON_ARGS_DOC = """
+    strategy : str in {'iteration', 'tolerance', 'callback'}
+        How the different precision solvers are called. Can be one of:
+        - ``'iteration'``: call the run method with max_iter number increasing
+        logarithmically to get more an more precise points.
+        - ``'tolerance'``: call the run method with tolerance decreasing
+        logarithmically to get more and more precise points.
+        - ``'callback'``: call the run method with a callback that will compute
+        the objective function on a logarithmic scale. After each iteration,
+        the callback should be called with the current iterate solution.
+    key_to_monitor : str (default: 'objective_value')
+        The objective to check for tracking progress.
+"""
+
+
 class StoppingCriterion():
-    """Class to check if we need to stop an algorithm.
+    f"""Class to check if we need to stop an algorithm.
 
     This base class will check for the timeout and the max_run.
     It should be sub-classed to check for the convergence of the algorithm.
@@ -42,33 +57,27 @@ class StoppingCriterion():
     **kwargs : dict
         All parameters passed when instantiating the StoppingCriterion. This
         will be used to re-create the criterion with extra arguments in the
-        runner.
-    strategy : str in {'iteration', 'tolerance', 'callback'}
-        How the different precision solvers are called. Can be one of:
-        - ``'iteration'``: call the run method with max_iter number increasing
-        logarithmically to get more an more precise points.
-        - ``'tolerance'``: call the run method with tolerance decreasing
-        logarithmically to get more and more precise points.
-        - ``'callback'``: call the run method with a callback that will compute
-        the objective function on a logarithmic scale. After each iteration,
-        the callback should be called with the current iterate solution.
-    key_to_monitor : str (default: 'objective_value')
-        The objective to check for tracking progress.
+        runner.{COMMON_ARGS_DOC}
     """
     kwargs = None
 
     def __init__(self, strategy=None, key_to_monitor='objective_value',
                  **kwargs):
 
-        assert strategy in STOPPING_STRATEGIES, (
-            f"strategy should be in {STOPPING_STRATEGIES}. Got '{strategy}'."
-        )
+        if strategy is not None:
+            assert strategy in SAMPLING_STRATEGIES, (
+                f"strategy should be in {SAMPLING_STRATEGIES}. "
+                f"Got '{strategy}'."
+            )
 
         self.kwargs = kwargs
         self.strategy = strategy
-        self.key_to_monitor = key_to_monitor
+        self.key_to_monitor = (
+            key_to_monitor if key_to_monitor.startswith('objective_')
+            else f'objective_{key_to_monitor}'
+        )
 
-    def get_runner_instance(self, max_runs=1, timeout=None, output=None,
+    def get_runner_instance(self, max_runs=1, timeout=None, terminal=None,
                             solver=None):
         """Copy the stopping criterion and set the parameters that depends on
         how benchopt runner is called.
@@ -80,11 +89,11 @@ class StoppingCriterion():
             the convergence curve.
         timeout : float
             The maximum duration in seconds of the solver run.
-        output : TerminalOutput or None
+        terminal : TerminalOutput or None
             Object to format string to display the progress of the solver.
         solver : BaseSolver
             The solver for which this stopping criterion is called. Used to get
-            overridden ``stopping_strategy`` and ``get_next``.
+            overridden ``sampling_strategy`` and ``get_next``.
 
         Returns
         -------
@@ -103,18 +112,17 @@ class StoppingCriterion():
                 "to implement a new StoppingCriterion."
             )
 
-        # Get strategy from solver
-        strategy = self.strategy
-        if solver is not None:
-            strategy = solver._solver_strategy
-        assert strategy in STOPPING_STRATEGIES, (
-            f"stopping_strategy should be in {STOPPING_STRATEGIES}. "
-            f"Got '{strategy}'."
-        )
+        if self.strategy is None:
+            self.strategy = solver.sampling_strategy or 'iteration'
+        elif solver is not None and solver.sampling_strategy is not None:
+            assert solver.sampling_strategy == self.strategy, (
+                'The strategy is set both in Solver.sampling_strategy and in '
+                'its criterion, and it does not match. Only set it once.'
+            )
 
         # Create a new instance of the class
         stopping_criterion = self.__class__(
-            strategy=strategy, key_to_monitor=self.key_to_monitor,
+            strategy=self.strategy, key_to_monitor=self.key_to_monitor,
             **self.kwargs,
         )
 
@@ -122,22 +130,23 @@ class StoppingCriterion():
         stopping_criterion.rho = RHO
         stopping_criterion.timeout = timeout
         stopping_criterion.max_runs = max_runs
-        stopping_criterion.output = output
+        stopping_criterion.terminal = terminal
         stopping_criterion.solver = solver
 
         # Override get_next_stop_val if ``get_next`` is implemented for solver.
         if hasattr(solver, 'get_next'):
-            assert (
-                callable(solver.get_next)
-                # and type(solver.get_next) == staticmethod
-            ), "if defined, get_next should be a static method of the solver."
+            if not callable(solver.get_next):
+                raise TypeError(
+                    f"`get_next` of Solver in {solver.__module__} "
+                    "must be callable."
+                )
+
             try:
-                solver.get_next(0)
+                solver.get_next(1)
             except TypeError:
                 raise ValueError(
-                    "get_next(0) throw a TypeError. Verify that `get_next` "
-                    "signature is get_next(stop_val) and that it is "
-                    "a staticmethod."
+                    "get_next(1) throw a TypeError. Verify that `get_next` "
+                    "signature is get_next(self, stop_val)"
                 )
 
             stopping_criterion.get_next_stop_val = solver.get_next
@@ -160,7 +169,7 @@ class StoppingCriterion():
         self.progress('initialization')
         return stop_val
 
-    def should_stop(self, stop_val, cost_curve):
+    def should_stop(self, stop_val, objective_list):
         """Base call to check if we should stop running a solver.
 
         This base call checks for the timeout and the max number of runs.
@@ -172,7 +181,7 @@ class StoppingCriterion():
         stop_val : int | float
             Corresponds to stopping criterion of the underlying algorithm, such
             as ``tol`` or ``max_iter``.
-        cost_curve : list of dict
+        objective_list : list of dict
             List of dict containing the values associated to the objective at
             each evaluated points.
 
@@ -184,16 +193,28 @@ class StoppingCriterion():
             Reason why the algorithm was stopped if stop is True.
         next_stop_val : int | float
             Next value for the stopping criterion. This value depends on the
-            stop strategy for the solver.
+            sampling strategy for the solver.
         """
+        # Check that the objective is compatible with the stopping_criterion
+        if self.key_to_monitor not in objective_list[0]:
+            key = self.key_to_monitor.replace("objective_", "")
+            raise ValueError(
+                "Objective.evaluate_result() should contain a key named "
+                f"'{key}' to be used with this stopping_criterion. The name of"
+                " this key can be changed via the 'key_to_monitor' parameter. "
+                f"Available keys are {list(objective_list[0].keys())}"
+            )
+
         # Modify the criterion state:
         # - compute the number of run with the curve. We need to remove 1 as
         #   it contains the initial evaluation.
         # - compute the delta_objective for debugging and stalled progress.
-        n_eval = len(cost_curve) - 1
-        objective = cost_curve[-1][self.key_to_monitor]
+        n_eval = len(objective_list) - 1
+        objective = objective_list[-1][self.key_to_monitor]
         delta_objective = self._prev_objective - objective
-        delta_objective /= abs(cost_curve[0][self.key_to_monitor])
+        first_objective = objective_list[0][self.key_to_monitor]
+        if first_objective != 0:
+            delta_objective /= abs(first_objective)
         self._prev_objective = objective
 
         # default value for is_flat
@@ -201,10 +222,12 @@ class StoppingCriterion():
 
         # check the different conditions:
         #     diverging / timeout / max_runs / stopping_criterion
+
         if math.isnan(objective) or delta_objective < -1e5:
             stop = True
             status = 'diverged'
-        elif self._deadline is not None and time.time() > self._deadline:
+
+        elif self._deadline is not None and time.time() >= self._deadline:
             stop = True
             status = 'timeout'
 
@@ -214,7 +237,7 @@ class StoppingCriterion():
         else:
             # Call the sub-class hook, used to check stopping criterion
             # on the curve.
-            stop, progress = self.check_convergence(cost_curve)
+            stop, progress = self.check_convergence(objective_list)
 
             # Display the progress if necessary
             progress = max(n_eval / self.max_runs, progress)
@@ -240,12 +263,12 @@ class StoppingCriterion():
 
         return stop, status, stop_val
 
-    def check_convergence(self, cost_curve):
+    def check_convergence(self, objective_list):
         """Check if the solver should be stopped based on the objective curve.
 
         Parameters
         ----------
-        cost_curve : list of dict
+        objective_list : list of dict
             List of dict containing the values associated to the objective at
             each evaluated points.
 
@@ -262,27 +285,32 @@ class StoppingCriterion():
 
     def debug(self, msg):
         """Helper to print debug messages."""
-        if self.output is not None:
-            self.output.debug(msg)
+        if self.terminal is not None:
+            self.terminal.debug(msg)
 
     def progress(self, progress):
         """Helper to print progress messages."""
-        if self.output is not None:
-            self.output.progress(progress)
+        if self.terminal is not None:
+            self.terminal.progress(progress)
 
     @staticmethod
     def _reconstruct(klass, kwargs, runner_kwargs):
         criterion = klass(**kwargs)
-        return criterion.get_runner_instance(**runner_kwargs)
+        if runner_kwargs:
+            return criterion.get_runner_instance(**runner_kwargs)
+        return criterion
 
     def __reduce__(self):
         kwargs = dict(
             strategy=self.strategy, **self.kwargs
         )
-        runner_kwargs = dict(
-            max_runs=self.max_runs, timeout=self.timeout,
-            output=self.output, solver=self.solver
-        )
+        if getattr(self, 'max_runs', None):
+            runner_kwargs = dict(
+                max_runs=self.max_runs, timeout=self.timeout,
+                terminal=self.terminal, solver=self.solver
+            )
+        else:
+            runner_kwargs = None
         return self._reconstruct, (self.__class__, kwargs, runner_kwargs)
 
     def get_next_stop_val(self, stop_val):
@@ -293,7 +321,7 @@ class StoppingCriterion():
 
 
 class SufficientDescentCriterion(StoppingCriterion):
-    """Stopping criterion based on sufficient descent.
+    f"""Stopping criterion based on sufficient descent.
 
     The solver will be stopped once successive evaluations do not make enough
     progress. The number of successive evaluation and the definition of
@@ -306,22 +334,10 @@ class SufficientDescentCriterion(StoppingCriterion):
         in the interval ``[-eps, eps]``.
     patience :  float (default: benchopt.stopping_criterion.PATIENCE)
         The solver is stopped after ``patience`` successive insufficient
-        updates.
-    strategy : str in {'iteration', 'tolerance', 'callback'}
-        (default: 'iteration')
-        How the different precision solvers are called. Can be one of:
-        - ``'iteration'``: call the run method with max_iter number increasing
-        logarithmically to get more an more precise points.
-        - ``'tolerance'``: call the run method with tolerance decreasing
-        logarithmically to get more and more precise points.
-        - ``'callback'``: call the run method with a callback that will compute
-        the objective function on a logarithmic scale. After each iteration,
-        the callback should be called with the current iterate solution.
-    key_to_monitor : str (default: 'objective_value')
-        The objective to check for tracking progress.
+        updates.{COMMON_ARGS_DOC}
     """
 
-    def __init__(self, eps=EPS, patience=PATIENCE, strategy='iteration',
+    def __init__(self, eps=EPS, patience=PATIENCE, strategy=None,
                  key_to_monitor='objective_value'):
         self.eps = eps
         self.patience = patience
@@ -334,12 +350,12 @@ class SufficientDescentCriterion(StoppingCriterion):
             key_to_monitor=key_to_monitor
         )
 
-    def check_convergence(self, cost_curve):
+    def check_convergence(self, objective_list):
         """Check if the solver should be stopped based on the objective curve.
 
         Parameters
         ----------
-        cost_curve : list of dict
+        objective_list : list of dict
             List of dict containing the values associated to the objective at
             each evaluated points.
 
@@ -353,9 +369,9 @@ class SufficientDescentCriterion(StoppingCriterion):
             that the solver has converged.
         """
         # Compute the current objective
-        objective = cost_curve[-1][self.key_to_monitor]
+        objective = objective_list[-1][self.key_to_monitor]
         delta_objective = self._objective - objective
-        delta_objective /= abs(cost_curve[0][self.key_to_monitor])
+        delta_objective /= abs(objective_list[0][self.key_to_monitor])
         self._objective = objective
 
         # Store only the last ``patience`` values for progress
@@ -373,7 +389,7 @@ class SufficientDescentCriterion(StoppingCriterion):
 
 
 class SufficientProgressCriterion(StoppingCriterion):
-    """Stopping criterion based on sufficient progress.
+    f"""Stopping criterion based on sufficient progress.
 
     The solver will be stopped once successive evaluations do not make enough
     progress. The number of successive evaluation and the definition of
@@ -386,22 +402,10 @@ class SufficientProgressCriterion(StoppingCriterion):
         smaller than ``eps``.
     patience :  float (default: benchopt.stopping_criterion.PATIENCE)
         The solver is stopped after ``patience`` successive insufficient
-        updates.
-    strategy : str in {'iteration', 'tolerance', 'callback'}
-        (default: 'iteration')
-        How the different precision solvers are called. Can be one of:
-        - ``'iteration'``: call the run method with max_iter number increasing
-        logarithmically to get more an more precise points.
-        - ``'tolerance'``: call the run method with tolerance decreasing
-        logarithmically to get more and more precise points.
-        - ``'callback'``: call the run method with a callback that will compute
-        the objective function on a logarithmic scale. After each iteration,
-        the callback should be called with the current iterate solution.
-    key_to_monitor : str (default: 'objective_value')
-        The objective to check for tracking progress.
+        updates.{COMMON_ARGS_DOC}
     """
 
-    def __init__(self, eps=EPS, patience=PATIENCE, strategy='iteration',
+    def __init__(self, eps=EPS, patience=PATIENCE, strategy=None,
                  key_to_monitor='objective_value'):
         self.eps = eps
         self.patience = patience
@@ -414,12 +418,12 @@ class SufficientProgressCriterion(StoppingCriterion):
             key_to_monitor=key_to_monitor
         )
 
-    def check_convergence(self, cost_curve):
+    def check_convergence(self, objective_list):
         """Check if the solver should be stopped based on the objective curve.
 
         Parameters
         ----------
-        cost_curve : list of dict
+        objective_list : list of dict
             List of dict containing the values associated to the objective at
             each evaluated points.
 
@@ -433,9 +437,11 @@ class SufficientProgressCriterion(StoppingCriterion):
             that the solver has converged.
         """
         # Compute the current objective and update best value
-        objective = cost_curve[-1][self.key_to_monitor]
+        objective = objective_list[-1][self.key_to_monitor]
         delta_objective = self._best_objective - objective
-        delta_objective /= abs(cost_curve[0][self.key_to_monitor])
+        first_objective = objective_list[0][self.key_to_monitor]
+        if first_objective != 0:
+            delta_objective /= abs(first_objective)
         self._best_objective = min(
             objective, self._best_objective
         )
@@ -452,3 +458,44 @@ class SufficientProgressCriterion(StoppingCriterion):
 
         progress = math.log(max(abs(delta), self.eps)) / math.log(self.eps)
         return False, progress
+
+
+class SingleRunCriterion(StoppingCriterion):
+    """Stopping criterion for single run solvers.
+
+    The solver will be stopped after one call to the objective.
+
+    Parameters
+    ----------
+    stop_val : int or float, (default: 1)
+        Value of ``stop_val`` with which the objective function will be called.
+        This value will be passed as ``n_iter`` or ``tol`` parameter for the
+        ``run`` method of solver with ``sampling_strategy`` respectively equals
+        to ``'iteration'`` or ``'tolerance'``, or the number of callback calls
+        minus one for the ``'callback'`` strategy.
+    """
+
+    def __init__(self, stop_val=1, strategy=None, *args, **kwargs):
+        # Necessary as the criterion is given a strategy argument when
+        # instanciated for an instance.
+        super().__init__(strategy=strategy, stop_val=stop_val)
+        self.stop_val = stop_val
+
+    def init_stop_val(self):
+        return self.stop_val
+
+    def get_runner_instance(self, max_runs=1, timeout=None, terminal=None,
+                            solver=None):
+
+        return super().get_runner_instance(1, timeout, terminal, solver)
+
+    def should_stop(self, stop_val, objective_list):
+        return True, 'done', stop_val
+
+
+class NoCriterion(StoppingCriterion):
+    """Run the solvers for a number of time fixed by max_iter and timeout.
+    """
+
+    def check_convergence(self, cost_curve):
+        return False, 0
