@@ -1,3 +1,4 @@
+import os
 import sys
 import builtins
 from contextlib import contextmanager
@@ -7,30 +8,6 @@ class FakeModule:
     __version__ = "1.0"
 
 
-@contextmanager
-def patch_benchmark(benchmark, component="objective", **updates):
-    "Patch a component of a given benchmark in the current interpreter"
-
-    if component == "objective":
-        obj = benchmark.get_benchmark_objective()
-    else:
-        raise ValueError("invalid component")
-
-    try:
-        na = object()
-        prev_value = {}
-        for k, v in updates.items():
-            prev_value[k] = getattr(obj, k, na)
-            setattr(obj, k, v)
-        yield
-    finally:
-        for k, v in prev_value.items():
-            if k is na:
-                delattr(obj, k)
-            else:
-                setattr(obj, k, v)
-
-
 @contextmanager
 def patch_import(**func_import):
     """Patch import in a context.
@@ -60,3 +37,16 @@ def patch_import(**func_import):
         yield
     finally:
         builtins.__import__ = builtins_import
+
+
+@contextmanager
+def patch_var_env(name, value):
+    try:
+        old_value = os.environ.get(name, None)
+        os.environ[name] = str(value)
+        yield
+    finally:
+        if old_value is None:
+            del os.environ[name]
+        else:
+            os.environ[name] = old_value
