@@ -2,9 +2,6 @@ import itertools
 from abc import abstractmethod
 
 
-from .dynamic_modules import _reconstruct_class
-
-
 class ParametrizedNameMixin():
     """Mixing for parametric classes representation and naming.
     """
@@ -19,11 +16,6 @@ class ParametrizedNameMixin():
         _parameters = next(product_param(self.parameters))
         _parameters.update(parameters)
         self._parameters = _parameters
-        if not hasattr(self, 'parameter_template'):
-            # sort parameters to make sure the representation is deterministic
-            self.parameter_template = ",".join([
-                f"{k}={_parameters[k]}" for k in sorted(_parameters)
-            ])
         for k, v in _parameters.items():
             if not hasattr(self, k):
                 setattr(self, k, v)
@@ -38,9 +30,20 @@ class ParametrizedNameMixin():
         try:
             obj = cls(**parameters)
             obj.save_parameters(**parameters)
-        except Exception:
-            print(cls)
+        except Exception as exception:
+            # get type (Dataset, Objective, or Solver) of class and its name
+            cls_type = cls.__base__.__name__
+            cls_type = cls_type.replace("Base", "")
+            cls_name = cls.name
+
+            # Extend exception error message
+            # TODO: use `add_note` when requiring python>=3.11
+            exception.args = (
+                f'Error when initializing {cls_type}: "{cls_name}". '
+                f'{". ".join(exception.args)}',
+            )
             raise
+
         return obj
 
     @property
@@ -53,7 +56,16 @@ class ParametrizedNameMixin():
         """Compute the parametrized name of the instance."""
         out = f"{self.name}"
         if len(self._parameters) > 0:
-            out += f"[{self.parameter_template}]".format(**self._parameters)
+            if not hasattr(self, 'parameter_template'):
+                # sort parameters to make sure the representation
+                # is deterministic
+                param_fmt = ",".join([
+                    f"{k}={self._parameters[k]}"
+                    for k in sorted(self._parameters)
+                ])
+            else:
+                param_fmt = self.parameter_template.format(**self._parameters)
+            out += f"[{param_fmt}]"
         return out
 
     @classmethod
@@ -61,15 +73,50 @@ class ParametrizedNameMixin():
         """Compute the parametrized name for a given set of parameters."""
         return str(cls.get_instance(**parameters))
 
-    @classmethod
-    def _reload_class(cls, pickled_module_hash=None):
+    @staticmethod
+    def _load_instance(benchmark_dir, cls_info, parameters):
+        # Make sure the running benchmark is set before loading the instance.
+        from benchopt.benchmark import Benchmark
+        Benchmark(benchmark_dir)
 
-        return _reconstruct_class(
-            cls._module_filename, cls._base_class_name,
-            cls._import_ctx.benchmark_dir,
-            pickled_module_hash=pickled_module_hash
+        # Load the dynamic class
+        from benchopt.utils.dynamic_modules import _reconstruct_class
+        klass = _reconstruct_class(benchmark_dir, *cls_info)
+
+        # Set the parameters of the parametrized class.
+        obj = klass.get_instance(**parameters)
+        return obj
+
+    def _get_mixin_args(self):
+        """Get the arguments necessary to reconstruct the instance."""
+
+        cls_info = (
+            str(self.__class__._module_filename),
+            self.__class__._base_class_name,
+            self.__class__._file_hash
         )
 
+        # Send the benchmark folder to the instance so it can access the config
+        from benchopt.benchmark import get_running_benchmark
+        benchmark_dir = get_running_benchmark().benchmark_dir
+
+        return str(benchmark_dir), cls_info, self._parameters
+
+    def __setstate__(self, state):
+        """Default setstate method to reconstruct the instance.
+
+        If `_get_state` is defined, this function should be overridden.
+        """
+        assert len(state) == 0, (
+            "If `_get_state` is defined, this function should be overridden."
+        )
+
+    def _get_state(self):
+        return {}
+
+    def __reduce__(self):
+        return self._load_instance, self._get_mixin_args(), self._get_state()
+
 
 def expand(keys, values):
     """Expand the multiple parameters for itertools product"""
@@ -105,14 +152,3 @@ def product_param(parameters):
     parameter_names = parameters.keys()
     return map(expand, itertools.repeat(parameter_names),
                itertools.product(*parameters.values()))
-
-
-def _list_all_parametrized_names(*parametrized_classes):
-    """List all names for parametrized classes."""
-    all_names = []
-    for cls in parametrized_classes:
-        for dataset_parameters in product_param(cls.parameters):
-            all_names.append(
-                cls._get_parametrized_name(**dataset_parameters)
-            )
-    return all_names
