@@ -10,6 +10,7 @@ import matplotlib.pyplot as plt
 from benchopt import run_benchmark
 from benchopt.benchmark import Benchmark
 from benchopt.plotting import plot_benchmark, PLOT_KINDS
+from benchopt.plotting.plot_objective_curve import reset_solver_styles_idx
 
 
 BENCHMARK_PATH = (
@@ -18,10 +19,11 @@ BENCHMARK_PATH = (
 
 
 try:
+
     save_file = run_benchmark(
-        Benchmark(BENCHMARK_PATH), ['sklearn[liblinear]', 'sklearn[newton-cg]',
-                                    'lightning'],
-        dataset_names=['Simulated*[n_features=500,n_samples=200]'],
+        BENCHMARK_PATH,
+        solver_names=['sklearn[liblinear]', 'sklearn[newton-cg]', 'lightning'],
+        dataset_names=["Simulated[n_features=500,n_samples=200]"],
         objective_filters=['L2 Logistic Regression[lmbd=1.0]'],
         max_runs=100, timeout=20, n_repetitions=15,
         plot_result=False, show_progress=True
@@ -37,6 +39,7 @@ except RuntimeError:
 
 
 kinds = list(PLOT_KINDS.keys())
+reset_solver_styles_idx()
 figs = plot_benchmark(save_file, benchmark=Benchmark(BENCHMARK_PATH),
                       kinds=kinds, html=False)
 plt.show()
