@@ -10,6 +10,7 @@ import matplotlib.pyplot as plt
 from benchopt import run_benchmark
 from benchopt.benchmark import Benchmark
 from benchopt.plotting import plot_benchmark, PLOT_KINDS
+from benchopt.plotting.plot_objective_curve import reset_solver_styles_idx
 
 
 BENCHMARK_PATH = Path().resolve().parent / 'benchmarks' / 'benchmark_lasso'
@@ -23,8 +24,8 @@ if not BENCHMARK_PATH.exists():
     )
 
 save_file = run_benchmark(
-    Benchmark(BENCHMARK_PATH),
-    ['Python-PGD[use_acceleration=False]', 'R-PGD'],
+    BENCHMARK_PATH,
+    solver_names=['Python-PGD[use_acceleration=False]', 'R-PGD'],
     dataset_names=["Simulated[n_features=5000,n_samples=100,rho=0]"],
     objective_filters=['*[fit_intercept=False,reg=0.5]'],
     max_runs=100, timeout=100, n_repetitions=5,
@@ -33,6 +34,8 @@ save_file = run_benchmark(
 
 
 kinds = list(PLOT_KINDS.keys())
-figs = plot_benchmark(save_file, benchmark=Benchmark(BENCHMARK_PATH),
-                      kinds=kinds, html=False)
+reset_solver_styles_idx()
+figs = plot_benchmark(
+    save_file, benchmark=Benchmark(BENCHMARK_PATH), kinds=kinds, html=False
+)
 plt.show()
